import {Injectable} from '@angular/core';
import {HttpClient, HttpHeaders} from '@angular/common/http';
 
@Injectable()
export class UserService {
 
  // http options used for making API calls
  private httpOptions: any;
 
  // the actual JWT token
  public token: string;
 
  // the token expiration date
  public token_expires: Date;
 
  // the username of the logged in user
  public username: string;
 
  // error messages received from the login attempt
  public errors: any = [];

  // Expiry in Seconds
  public expiry: number;
 
  constructor(private http: HttpClient) {
    this.httpOptions = {
      headers: new HttpHeaders({'Content-Type': 'application/json'})
    };
    // Default Expiry is 24 Hours.  Override if you want it more secure
    this.expiry = 3600 * 24;
  }
 
  // Uses http.post() to get an auth token from djangorestframework-jwt endpoint
  public login(user) {
    this.http.post('/api-token-auth/', JSON.stringify(user), this.httpOptions).subscribe(
      data => {
        this.updateData(data['token']);
      },
      err => {
        this.errors = err['error'];
      }
    );
  }
 
  // Refreshes the JWT token, to extend the time the user is logged in
  public refreshToken() {
    this.http.post('/api-token-refresh/', JSON.stringify({token: this.token}), this.httpOptions).subscribe(
      data => {
        this.updateData(data['token']);
      },
      err => {
        this.errors = err['error'];
      }
    );
  }

  // Returns True if Logged in.  Also refreshs if 20% of time
  public isLoggedIn() {
    var timeLeft: number;
      if (this.token_expires != null) {
        if (Date.now().valueOf() >= this.token_expires.valueOf()) {
          // Token Expired - Make sure logged out.
          this.logout();
          return false;
        } else {
          timeLeft = this.token_expires.valueOf() - Date.now();
          if (timeLeft < (this.expiry / 20)) {
            // Automatically Refresh
            this.refreshToken();
          }
          // User is Logged In
          return true;
        }
      } else {
        // No Token - Not logged in
        return false;
      }
  }
 
  public logout() {
    this.token = null;
    this.token_expires = null;
    this.username = null;
  }
 
  private updateData(token) {
    this.token = token;
    this.errors = [];
 
    // decode the token to read the username and expiration timestamp
    const token_parts = this.token.split(/\./);
    const token_decoded = JSON.parse(window.atob(token_parts[1]));
    this.token_expires = new Date(token_decoded.exp * 1000);
    this.username = token_decoded.username;
  }
 
}